package com.kexun.generate;

import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.generator.AutoGenerator;
import com.baomidou.mybatisplus.generator.InjectionConfig;
import com.baomidou.mybatisplus.generator.config.*;
import com.baomidou.mybatisplus.generator.config.rules.DateType;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.baomidou.mybatisplus.generator.engine.FreemarkerTemplateEngine;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

/**
 * Description:
 *
 * @author lgs
 * @date 2023/05/05
 */
@Slf4j
public class MybatisPlusCodeGenerator {

    public static void main(String[] args) {
        try {

            //====================配置变量区域=====================//
            //生成文件的作者，可以不填
            String author="lgs";
            //生成的entity、controller、service等包所在的公共上一级包路径全限定名
            String moduleName="admin-api";
            String path = System.getProperty("user.dir")+"/"+moduleName;

            log.info("path={}",path);
            //包名
            String packageName = "com.kexun";
            //====================配置变量区域=====================//

            //数据库配置
            String url="jdbc:mysql://39.101.189.130:33306/cr_loan?useSSL=false&characterEncoding=utf8&allowPublicKeyRetrieval=true";

            String driverClassName="com.mysql.jdbc.Driver";
            String username="root";
            String password="1q2w3e4r";
            //表名，多个使用,分隔
            String tableNames="report_file";
            //====================定义常量=========================//
            // 代码生成器
            AutoGenerator generator = new AutoGenerator();

            // =======1、数据源配置start==============
            DataSourceConfig dataSourceConfig = new DataSourceConfig();
            //连接URL
            dataSourceConfig.setUrl(url);
            //数据库类型
            dataSourceConfig.setDbType(DbType.MYSQL);
            //驱动名称
            dataSourceConfig.setDriverName(driverClassName);
            //数据库连接用户名
            dataSourceConfig.setUsername(username);
            //数据库连接密码
            dataSourceConfig.setPassword(password);
            generator.setDataSource(dataSourceConfig);
            //=======1、数据源配置 end==============
            //=======2、数据库表配置 start==========
            StrategyConfig strategyConfig = new StrategyConfig();
            //是否大写命名
            strategyConfig.setCapitalMode(false);
            //是否跳过视图
            strategyConfig.setSkipView(false);
            //数据库表映射到实体的命名策略
            //no_change,underline_to_camel
            strategyConfig.setNaming(NamingStrategy.underline_to_camel);
            //数据库表字段映射到实体的命名策略, 未指定按照 naming 执行
            strategyConfig.setColumnNaming(NamingStrategy.underline_to_camel);
            //表前缀
            strategyConfig.setTablePrefix("cr_");
            //字段前缀
            //strategyConfig.setFieldPrefix("");
            //自定义继承的 Entity 类全称，带包名
            strategyConfig.setSuperEntityClass("com.kexun.entity.BaseEntity");

            //自定义基础的 Entity 类，公共字段 可填写多个字段，
            strategyConfig.setSuperEntityColumns("id","tenant_id","revison","create_by","create_time","update_by","update_time","is_deleted","remarks");

            //自定义继承的 Mapper 类全称，带包名
            //strategyConfig.setSuperMapperClass("");

            //自定义继承的 Service 类全称，带包名
            //strategyConfig.setSuperServiceClass("");

            //自定义继承的 ServiceImpl 类全称，带包名
            //strategyConfig.setSuperServiceImplClass("");

            //自定义继承的 Controller 类全称，带包名
            //strategyConfig.setSuperControllerClass("")

            //【实体】是否为 lombok 模型（默认 false）
            strategyConfig.setEntityLombokModel(true);
            //生成 @RestController 控制器
            strategyConfig.setRestControllerStyle(true);
            //是否生成实体时，生成字段注解
            strategyConfig.setEntityTableFieldAnnotationEnable(true);

            //乐观锁属性名称
            //strategyConfig.setVersionFieldName("");
            //需要包含的表名，当 enableSqlFilter 为 false 时，允许正则表达式（与 exclude 二选一配置）
            strategyConfig.setInclude(tableNames);

            generator.setStrategy(strategyConfig);
            //=======2、数据库表配置 end==========
            //=======3、全局配置 start===========
            GlobalConfig globalConfig = new GlobalConfig();

            //生成文件的输出目录【默认 D 盘根目录】
            globalConfig.setOutputDir(path +"/src/main/java");
            log.info("outPutDir={}",globalConfig.getOutputDir());
            //是否覆盖已有文件，默认false
            globalConfig.setFileOverride(false);
            //是否打开输出目录
            globalConfig.setOpen(false);
            //是否在 xml 中添加二级缓存配置 默认false
            //globalConfig.setEnableCache(false);
            //开发人员
            globalConfig.setAuthor(author);
            //开启swagger2模式
            globalConfig.setSwagger2(true);
            //开启 ActiveRecord 模式
            //globalConfig.setActiveRecord(true);
            //开启 BaseResultMap
            globalConfig.setBaseResultMap(true);
            //开启 baseColumnList
            globalConfig.setBaseColumnList(true);
            //时间类型对应策略
            globalConfig.setDateType(DateType.ONLY_DATE);
            //实体命名方式
            globalConfig.setEntityName("%sEntity");
            //mapper命名方式
            globalConfig.setMapperName("%sMapper");
            //Mapper xml 命名方式
            globalConfig.setXmlName("%sMapper");
            //service命名方式
            globalConfig.setServiceName("%sService");
            //service impl 命名方式
            globalConfig.setServiceImplName("%sServiceImpl");
            //controller 命名方式
            globalConfig.setControllerName("%sController");
            //分配ID (主键类型为number或string）
            globalConfig.setIdType(IdType.ASSIGN_ID);
            generator.setGlobalConfig(globalConfig);
            //=======3、全局配置 end===========

            //=======4、自定义配置 start=======
            // 自定义配置,自定义变量配置
            /** 自定义 生成类**/
            Map<String,Object> customFileMap = new HashMap<>();
            /**PO实体**/
            customFileMap.put("po"+ File.separator+"%sPO.java", "/templates/PO.java.vm");
            /**Vo实体**/
            customFileMap.put("vo"+File.separator+"%sVO.java", "/templates/VO.java.vm");
            /**DTO实体**/
            customFileMap.put("dto"+File.separator+"%sDTO.java", "/templates/DTO.java.vm");

            InjectionConfig injectionConfig = new InjectionConfig() {
                @Override
                public void initMap() {
                    //this.setMap(customFileMap);
                }
            };
            //generator.setCfg(injectionConfig);

            //=======4、自定义配置end  =======
            //=======5、包配置 start  =======
            PackageConfig packageConfig = new PackageConfig();
            //父包名 如果为空，将下面子包名必须写全部， 否则就只需写子包名
            packageConfig.setParent(packageName);
            log.info("parent={}",packageConfig.getParent());
            //父包模块名 默认值无
            //packageConfig.setModuleName(scanner("模块名"));
            //Controller 包名
            packageConfig.setController("controller");
            //Entity 包名
            packageConfig.setEntity("entity");
            //Service 包名
            packageConfig.setService("service");
            //Service Impl 包名
            packageConfig.setServiceImpl("service.impl");
            //Mapper 包名
            packageConfig.setMapper("mapper");
            //Mapper XML 包名
            //路径配置信息
            // 自定义生成路径
            Map<String,String> map = new HashMap<>();
            map.put("xml_path",path + "/src/main/resources/mapper");
            map.put("entity_path",path + "/src/main/java/com/kexun/entity");
            map.put("mapper_path",path + "/src/main/java/com/kexun/mapper");
            map.put("service_path",path + "/src/main/java/com/kexun/service");
            map.put("service_impl_path",path + "/src/main/java/com/kexun/service/impl");
            map.put("controller_path",path + "/src/main/java/com/kexun/controller");
            packageConfig.setPathInfo(map);

            generator.setPackageInfo(packageConfig);
            //=======5、包配置 end  =======

            //=======6、自定义代码模板======
            TemplateConfig templateConfig = new TemplateConfig();
            //controller模板
            //templateConfig.setController("");

            //generator.setTemplate(templateConfig);

            generator.setTemplateEngine(new FreemarkerTemplateEngine());
            generator.execute();
            log.info("生成完成");
        }catch (Exception e){
            e.printStackTrace();
        }

    }
}
