package com.kexun.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.kexun.constant.BaseConsts;
import com.kexun.entity.*;
import com.kexun.exception.RRException;
import com.kexun.hander.PageBean;
import com.kexun.mapper.CooperatorMapper;
import com.kexun.mapper.FieldDefineMapper;
import com.kexun.mapper.FieldTemplateMapper;
import com.kexun.model.ro.*;
import com.kexun.model.vo.FieldVO;
import com.kexun.model.vo.FiledTemplateVO;
import com.kexun.service.*;
import com.kexun.utils.PageUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;


import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 */
@Service
public class FieldTemplateServiceImpl extends ServiceImpl<FieldTemplateMapper, FieldTemplateEntity> implements FieldTemplateService {
    @Autowired
    FieldDefineService fieldDefineService;
    @Autowired
    FieldTemplateMapper fieldTemplateMapper;

    @Autowired
    FieldDefineMapper fieldDefineMapper;
    @Autowired
    FieldTemplateRefService fieldTemplateRefService;



    /**
     * 分页查询模板字段
     * @param filedTemplateRO
     * @return
     */
    @Override
    public IPage<FieldTemplateEntity> listPage(FiledTemplateRO filedTemplateRO) {
        PageBean<FieldTemplateEntity> page = new PageBean<>(PageUtils.getPageNo(filedTemplateRO.getPageNo()), PageUtils.getPageSize(filedTemplateRO.getPageSize()));
        QueryWrapper queryWrapper = new QueryWrapper();
        //字段模板编号
        if (!StringUtils.isEmpty(filedTemplateRO.getTemplateNo())) {
            queryWrapper.eq("templateNo", filedTemplateRO.getTemplateNo());
        }
        //字段模板名称
        if (!StringUtils.isEmpty(filedTemplateRO.getTemplateName())) {
            queryWrapper.like("templateName", filedTemplateRO.getTemplateName());
        }
        IPage<FieldTemplateEntity> pageList = fieldTemplateMapper.selectPage(page, queryWrapper);
        return pageList;
    }

    public List<FieldTemplateEntity> selectPage() {
//        System.out.println("aaa");
        List<FieldTemplateEntity> l = fieldTemplateMapper.selectAll();
        return l;
    }

    public List<FieldTemplateEntity> selectTemplate(String templateName) {
        List<FieldTemplateEntity> l = fieldTemplateMapper.selectTemplate(templateName);
        return l;
    }


    public FieldTemplateEntity findByID(long id) {
        QueryWrapper<FieldTemplateEntity> wrapper=new QueryWrapper<FieldTemplateEntity>().eq("id",id);
        return getOne(wrapper);
    }


    /**
     * 保存模板字段
     *
     * @param filedTemplateCreateRO
     */
    @Override
    public void save(FiledTemplateCreateRO filedTemplateCreateRO) {
        if (null != filedTemplateCreateRO) {
            FieldTemplateEntity fieldTemplateEntity = new FieldTemplateEntity();
            fieldTemplateEntity.setStatus(BaseConsts.STATUS_ON);
            long time1 = System.currentTimeMillis();
            filedTemplateCreateRO.setTemplateNo(Long.toString(time1));

            BeanUtils.copyProperties(filedTemplateCreateRO, fieldTemplateEntity);
            fieldTemplateMapper.insert(fieldTemplateEntity);
            Long id = fieldTemplateEntity.getId();
            FieldTemplateEntity templateEntity = getById(id);


            //模板字段关系表
            List <String> fieldSelected = filedTemplateCreateRO.getFieldSelected();
            List <String> fieldSelected2 = filedTemplateCreateRO.getFieldSelected2();
            List<FieldTemplateRefEntity> fieldTemplateRefList = new ArrayList<>();

            List<FieldDefineEntity> fieldBaseDefineList = fieldDefineMapper.selectBatchIds(fieldSelected);
            List<FieldDefineEntity> fieldDeriveDefineList = fieldDefineMapper.selectBatchIds(fieldSelected2);

            for (int i=0;i<fieldSelected.size();i++){
                Long fid = Long.parseLong(fieldSelected.get(i));
//                FieldDefineEntity fieldDefine = fieldDefineService.findByID(fid);
                FieldTemplateRefEntity fieldTemplateRefEntity = new FieldTemplateRefEntity();
                fieldTemplateRefEntity.setIsDeleted(false);
                fieldTemplateRefEntity.setFieldDefineId(fieldBaseDefineList.get(i).getId());
                fieldTemplateRefEntity.setFieldNo(fieldBaseDefineList.get(i).getFieldNo());
                fieldTemplateRefEntity.setName(fieldBaseDefineList.get(i).getName());
                fieldTemplateRefEntity.setType(fieldBaseDefineList.get(i).getCategory());
                fieldTemplateRefEntity.setFieldTemplateId(id);
//                fieldTemplateRefEntity.setType("1");
                fieldTemplateRefEntity.setTemplateNo(fieldTemplateEntity.getTemplateNo());
                fieldTemplateRefList.add(fieldTemplateRefEntity);
            }
            for (int i=0;i<fieldSelected2.size();i++){
                Long fid = Long.parseLong(fieldSelected2.get(i));
//                FieldDefineEntity fieldDefine = fieldDefineService.findByID(fid);
                FieldTemplateRefEntity fieldTemplateRefEntity = new FieldTemplateRefEntity();
                fieldTemplateRefEntity.setIsDeleted(false);
                fieldTemplateRefEntity.setFieldDefineId(fieldDeriveDefineList.get(i).getId());
                fieldTemplateRefEntity.setFieldNo(fieldDeriveDefineList.get(i).getFieldNo());
                fieldTemplateRefEntity.setType(fieldDeriveDefineList.get(i).getCategory());

                fieldTemplateRefEntity.setName(fieldDeriveDefineList.get(i).getName());
                fieldTemplateRefEntity.setFieldTemplateId(id);
//                fieldTemplateRefEntity.setType("2");

                fieldTemplateRefEntity.setTemplateNo(fieldTemplateEntity.getTemplateNo());
                fieldTemplateRefList.add(fieldTemplateRefEntity);
            }
            fieldTemplateRefService.saveBatch(fieldTemplateRefList);

        }
    }

    /**
     * 保存或者修改字段定义
     * @param filedTemplateUpdateRO
     */
    @Override
    public void newUpdate(FiledTemplateUpdateRO filedTemplateUpdateRO) {
        Long id = filedTemplateUpdateRO.getId();
        FieldTemplateEntity fieldTemplateEntity = new FieldTemplateEntity();
        fieldTemplateEntity.setTemplateNo(filedTemplateUpdateRO.getTemplateNo());
        fieldTemplateEntity.setTemplateName(filedTemplateUpdateRO.getTemplateName());
        BeanUtils.copyProperties(filedTemplateUpdateRO,fieldTemplateEntity);
        updateById(fieldTemplateEntity);

        List <String> fieldBaseSelectedOld = filedTemplateUpdateRO.getFieldBaseSelectedOld();
        for (int i=0;i<fieldBaseSelectedOld.size();i++){
            Long foid = Long.parseLong(fieldBaseSelectedOld.get(i));

            fieldTemplateRefService.removeById(foid);
        }
        List <String> fieldDeriveSelectedOld = filedTemplateUpdateRO.getFieldDeriveSelectedOld();
        for (int i=0;i<fieldDeriveSelectedOld.size();i++){
            Long foid = Long.parseLong(fieldDeriveSelectedOld.get(i));

            fieldTemplateRefService.removeById(foid);
        }
        List <String> fieldBaseSelected = filedTemplateUpdateRO.getFieldBaseSelected();
        List<FieldTemplateRefEntity> fieldTemplateRefList = new ArrayList<>();

        for (int i=0;i<fieldBaseSelected.size();i++){
            Long fid = Long.parseLong(fieldBaseSelected.get(i));
            FieldDefineEntity fieldDefine = fieldDefineService.findByID(fid);
            FieldTemplateRefEntity fieldTemplateRefEntity = new FieldTemplateRefEntity();
            fieldTemplateRefEntity.setIsDeleted(false);
            fieldTemplateRefEntity.setFieldDefineId(fieldDefine.getId());
            fieldTemplateRefEntity.setFieldNo(fieldDefine.getFieldNo());
            fieldTemplateRefEntity.setName(fieldDefine.getName());
            fieldTemplateRefEntity.setType(fieldDefine.getCategory());
            fieldTemplateRefEntity.setFieldTemplateId(id);
            fieldTemplateRefEntity.setTemplateNo(fieldTemplateEntity.getTemplateNo());
            fieldTemplateRefList.add(fieldTemplateRefEntity);
        }

        List <String> fieldDeriveSelected = filedTemplateUpdateRO.getFieldDeriveSelected();

        for (int i=0;i<fieldDeriveSelected.size();i++){
            Long fid = Long.parseLong(fieldDeriveSelected.get(i));
            FieldDefineEntity fieldDefine = fieldDefineService.findByID(fid);
            FieldTemplateRefEntity fieldTemplateRefEntity = new FieldTemplateRefEntity();
            fieldTemplateRefEntity.setIsDeleted(false);
            fieldTemplateRefEntity.setFieldDefineId(fieldDefine.getId());
            fieldTemplateRefEntity.setFieldNo(fieldDefine.getFieldNo());
            fieldTemplateRefEntity.setName(fieldDefine.getName());
            fieldTemplateRefEntity.setType(fieldDefine.getCategory());
            fieldTemplateRefEntity.setFieldTemplateId(id);
            fieldTemplateRefEntity.setTemplateNo(fieldTemplateEntity.getTemplateNo());
            fieldTemplateRefList.add(fieldTemplateRefEntity);
        }
        fieldTemplateRefService.saveBatch(fieldTemplateRefList);

    }

    /**
     * 修改模板
     *
     * @param filedTemplateUpdateRO
     */
    @Override
    public void update(FiledTemplateUpdateRO filedTemplateUpdateRO) {
        Long id = filedTemplateUpdateRO.getId();
        FieldTemplateEntity fieldTemplateSelect = fieldTemplateMapper.selectById(id);
        if (null != fieldTemplateSelect) {
            List<FiledUpdateRO> filedList = filedTemplateUpdateRO.getFieldSelected();
            //先删除，后insert
            List<FieldTemplateRefEntity> filedTemplateList = new ArrayList<>();
            for (FiledUpdateRO filedUpdateRO : filedList) {
                FieldTemplateRefEntity fieldTemplateRefEntity = new FieldTemplateRefEntity();
                fieldTemplateRefEntity.setFieldTemplateId(id);
                fieldTemplateRefEntity.setTemplateNo(filedTemplateUpdateRO.getTemplateNo());
                fieldTemplateRefEntity.setFieldDefineId(filedUpdateRO.getId());
                fieldTemplateRefEntity.setFieldNo(filedUpdateRO.getFieldNo());
                fieldTemplateRefEntity.setName(filedUpdateRO.getName());
                fieldTemplateRefEntity.setType(filedUpdateRO.getType());
                filedTemplateList.add(fieldTemplateRefEntity);
            }
            fieldTemplateRefService.deleteByTemplateId(id);
            fieldTemplateRefService.saveBatch(filedTemplateList);
            //修改模板字段表
            FieldTemplateEntity fieldTemplateEntity = new FieldTemplateEntity();
            fieldTemplateEntity.setTemplateNo(filedTemplateUpdateRO.getTemplateNo());
            fieldTemplateEntity.setTemplateName(filedTemplateUpdateRO.getTemplateName());
            fieldTemplateEntity.setId(id);
            updateById(fieldTemplateEntity);

        } else {
            throw new RRException("查询不到字段模板记录", -1);
        }

    }

    /**
     * @param id     主键
     * @param status 状态
     */
    @Override
    public void updateOnOff(Long id, String status) {
        fieldTemplateMapper.updateStatus(id, status);
    }

    /**
     * 删除ID
     *
     * @param id 主键
     */
    @Override
    public void delete(Long id) {
        //删除字段模板信息表
        removeById(id);
        //删除字段模板关联关系表
        fieldTemplateRefService.deleteByTemplateId(id);
    }

    /**
     * 查询字段详情
     *
     * @param id 主键
     * @return
     */
    @Override
    public FiledTemplateVO detail(Long id) {
        FieldTemplateEntity fieldTemplateEntity = getById(id);

        FiledTemplateVO filedTemplateVO = new FiledTemplateVO();
        BeanUtils.copyProperties(fieldTemplateEntity, filedTemplateVO);

        QueryWrapper queryWrapper = new QueryWrapper();
        //字段模板编号
        queryWrapper.eq("field_template_id", id);
        List<FieldTemplateRefEntity> list = fieldTemplateRefService.list(queryWrapper);

        List<FieldVO> fieldVOList = new ArrayList<>();
        for (FieldTemplateRefEntity fieldTemplateRefEntity : list) {
            FieldVO fieldVO = new FieldVO();
            BeanUtils.copyProperties(fieldTemplateRefEntity, fieldVO);
            fieldVOList.add(fieldVO);
        }
        filedTemplateVO.setFiledList(fieldVOList);
        return filedTemplateVO;
    }
}




