package com.kexun.utils;

import org.joda.time.*;
import org.springframework.util.StringUtils;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;
import java.util.Date;

/**
 * Description:日期出来Utils
 *
 * @author lgs
 * @date 2023/05/29
 */
public class DateUtils {

    public static final String DATE_YYYY_MM_DD_HH_MM_SS = "yyyy-MM-dd HH:mm:ss";
    public static final String YYYYMMDD = "yyyyMMdd";
    public static final String YYYY_MM = "yyyy-MM";
    public static final String YYYY_MM_DD = "yyyy-MM-dd";
    public static final String YYYYMMDDHHMMSS = "yyyyMMddHHmmss";
    public static final String DATE_YYYY_MM_DD_HH_MM = "yyyy-MM-dd HH:mm";
    public static final String DATE_HH_MM_SS = "HH:mm:ss";
    public static final String DATE_YYYY_MM_DD_HH = "yyyyMMddHH";
    public static final String DATE_HH_MM = "HH:mm";

    public static final String UNIT_MOUTHS = "M";
    public static final String UNIT_DAYS = "D";
    public static final String UNIT_YEARS = "Y";


    /**
     * 将带有T的时间字符串转换成yyyy-MM-dd HH:mm:ss
     *
     * @param strDate 带有T的时间
     * @return
     */
    public static String convertTDate(String strDate) {
        String str = "";
        try {
            String fmt = "yyyy-MM-dd HH:mm:ss";
            strDate = strDate.replace("T", " ");
            DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(fmt);
            return dateTimeFormatter.format(dateTimeFormatter.parse(strDate));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return str;
    }

    /**
     * 字符串转日期
     *
     * @param paraDate 字符日期
     * @param pattern  匹配类型
     * @return Date
     */
    public static Date convertStrToDate(String paraDate, String pattern) {
        DateFormat format = new SimpleDateFormat(pattern);
        Date date = null;
        if (!StringUtils.isEmpty(paraDate)) {
            try {
                date = format.parse(paraDate);
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }
        return date;
    }

    /**
     * 根据类型获取日期之间的天数，月份，年份
     *
     * @param beginDate 开始日期
     * @param endDate   结束日期
     * @param unit      单位
     * @return 间隔
     */
    public static int getSpaceByUnit(Date beginDate, Date endDate, String unit) {
        DateTime beginDateTime = new DateTime(beginDate);
        DateTime endDateTime = new DateTime(endDate);
        int units = 0;
        switch (unit) {
            case UNIT_DAYS:
                Days daysBetween = Days.daysBetween(beginDateTime, endDateTime);
                units = daysBetween.getDays();
                break;
            case UNIT_MOUTHS:
                Months months = Months.monthsBetween(beginDateTime, endDateTime);
                units = months.getMonths();
                break;
            case UNIT_YEARS:
                Years years = Years.yearsBetween(beginDateTime, endDateTime);
                units = years.getYears();
                break;
            default:
                Days dafaults = Days.daysBetween(beginDateTime, endDateTime);
                units = dafaults.getDays();
                break;
        }
        return units;
    }

    /**
     * 根据日期获取上个月最后一天日期
     *
     * @param date 日期
     * @param step 步长
     * @param type 类型
     * @return 上个月最后一天日期
     */
    public static Date lastMonthDate(Date date, int step, String type) {
        LocalDate localDate = new LocalDate(date);
        LocalDate resulDate = null;
        switch (type) {
            case "M":
                resulDate = localDate.plusMonths(-step);
                break;
            case "Y":
                resulDate = localDate.plusYears(-step);
                break;
            case "D":
                resulDate = localDate.plusDays(-step);
            default:
                resulDate = localDate;
        }
        return resulDate.toDate();
    }

    /**
     * 获取指定日期的年月日
     *
     * @param date
     * @return
     */
    public static Integer getUnitDate(Date date, String unit) {
        Integer reslut = null;
        LocalDate localDate = new LocalDate(date);
        switch (unit) {
            case "Y":
                reslut = localDate.getYear();
                break;
            case "M":
                reslut = localDate.getMonthOfYear();
                break;
            case "D":
                reslut = localDate.getDayOfMonth();
                break;
        }
        return reslut;
    }

    public static void main(String[] args) {
        Date beginDate = new Date();
        DateTime beginDateTime = new DateTime(beginDate);
        Integer y = getUnitDate(beginDate, "Y");
        System.out.println(y);

    }

}
